<?php
/**
* Класс для работы с почтой при помощи протокола pop3.
*
* @author       andi <ab@cdef.ru>
* @copyright    © Centurys ent., 2007
* @link         http://a-panov.ru/
* @since        1.0b
* @package      public scripts
* @access       public
*/

/**
* cpop3 класс.
*
* Класс служит для работы с почтовым ящиком средствами протокола pop3.
*/
class canpop3 {

    /**
    * Текст ошибки подключения.
    *
    * @since 1.0
    * @var string $error
    */
    public $error  = '';
    
    /**
    * Количество писем в ящике.
    *
    * @since 1.0
    * @var int $count
    */
    public $count  = -1;
    
    /**
    * Размер писем в ящике.
    *
    * @since 1.0
    * @var int $size
    */
    public $size   = -1;
    
    /**
    * Идентификатор соединения.
    *
    * @since 1.0
    * @var res $_classes
    */
    private $_fs = NULL;
    
    /**
    * Конструктор класса.
    *
    * @since 1.0
    * @param string $host хост для работы с pop3
    * @param string $user учётная запись
    * @param string $pass пароль
    * @param int $ssl использовать ли защищённое соединение
    * @param int $port номер порта
    */
    function __construct($host, $user, $pass, $ssl = FALSE, $port = FALSE) {
        if (FALSE === $port) {
            $port = $ssl ? 995 : 110;
        }
        $fs = fsockopen(($ssl ? 'ssl://' : '') . $host, $port, $errno, $errstr, 5);
        
        # при ошибке дальше делать нечего
        if ($errno) {
            $this->error = $errstr . " ($errno)";
            return;
        }
        
        $this->_fs =& $fs;
        $status = $this->_get();
        if ($status === FALSE || $status[2] != 'ok') {
            $this->error = empty($status[3]) ? 'Undefined error' : $status[3];
            fclose($fs);
            $this->_fs = NULL;
            return;
        }
        
        # передача имени пользователя
        fputs($fs, 'USER ' . $user . "\r\n");
        $status = $this->_get();
        if ($status[2] != 'ok') {
            $this->error = $status[3];
            fclose($fs);
            $this->_fs = NULL;
            return;
        }
        
        # передача пароля
        fputs($fs, 'PASS ' . $pass . "\r\n");
        $status = $this->_get();
        if ($status[2] != 'ok') {
            $this->error = $status[3];
            fclose($fs);
            $this->_fs = NULL;
            return;
        }
        
        # проверим, отвечается ли сервер
        if ( ! $this->noop()) {
            fclose($fs);
            $this->_fs = NULL;
            $this->error = 'POP3-server not request';
        }
        
        # получение статистики
        $this->stat();
    }
    
    /**
    * Проверка ответа сервера.
    *
    * Всегда возвращает +OK
    */
    function noop() {
        if (empty($this->_fs)) return FALSE;
        $this->error = '';
        
        fputs($this->_fs, "NOOP\r\n");
        $status = $this->_get();
        if (isset($status[2]) && $status[2] == 'ok') return TRUE;
        else {
             $this->error = isset($status[3]) ? $status[3] : 'Undefined error';
             return FALSE;
        }
    }
    
    /**
    * Откат транзакций внутри текущей сессии.
    */
    function rset() {
        if (empty($this->_fs)) return FALSE;
        $this->error = '';
        
        fputs($this->_fs, "RSET\r\n");
        $status = $this->_get();
        if (isset($status[2]) && $status[2] == 'ok') return TRUE;
        else {
             $this->error = isset($status[3]) ? $status[3] : 'Undefined error';
             return FALSE;
        }
    }
    
    /**
    * Пометить сообщение как удалённое.
    */
    function delete($id) {
        if (empty($this->_fs)) return FALSE;
        $this->error = '';
        
        fputs($this->_fs, "DELE " . $id . "\r\n");
        $status = $this->_get();
        if (isset($status[2]) && $status[2] == 'ok') return TRUE;
        else {
             $this->error = isset($status[3]) ? $status[3] : 'Undefined error';
             return FALSE;
        }
    }
    
    /**
    * Завершение сессии с принятием транзакций.
    *
    * Если эту функцию не вызвать, все удаления будут отменены.
    */
    function quit() {
        if (empty($this->_fs)) return FALSE;
        $this->error = '';
        
        fputs($this->_fs, "QUIT\r\n");
        $status = $this->_get();
        if (isset($status[2]) && $status[2] == 'ok') {
            fclose($this->_fs);
            $this->_fs = NULL;
            return TRUE;
        } else {
             $this->error = isset($status[3]) ? $status[3] : 'Undefined error';
             return FALSE;
        }
    }
    
    /**
    * Возвращает количество писем и их размер.
    *
    * +OK 7 27988
    */
    function stat() {
        if (empty($this->_fs)) return FALSE;
        $this->error = '';
        
        fputs($this->_fs, "STAT\r\n");
        $status = $this->_get();
        if (isset($status[2]) && $status[2] == 'ok') {
            list($this->count, $this->size) = explode(' ', $status[3]);
            return TRUE;
        } else {
            $this->error = isset($status[3]) ? $status[3] : 'Undefined error';
            return FALSE;
        }
    }
    
    /**
    * Возвращает информацию о сообщениях.
    *
    * Если был передан аргумент, возвращается информация об указанном сообщении,
    * иначе — обо всех.
    * пример:
    * +OK 7 messages:
    * 1 4486
    * 2 3514
    * 3 3691
    * 4 2826
    * 5 4516
    * 6 4141717
    * 7 4478
    * .
    */
    function lists($num = FALSE) {
        if (empty($this->_fs)) return FALSE;
        $this->error = '';
        
        $arr = FALSE;
        
        if ($num === FALSE) {
            fputs($this->_fs, "LIST\r\n");
            $status = $this->_get();
            if (isset($status[2]) && $status[2] == 'ok') {
                $t = $this->_get_other();
                preg_match('!([0-9]+) messages:!i', $status[3], $result);
                $arr = array();
                if (isset($result[1]) && $result[1] == 0) ;
                else {
                    $t = preg_replace('!\r?\n!', "\n", trim($t));
                    foreach (explode("\n", $t) as $v) {
                        list($id, $size) = explode(' ', $v);
                        $arr[$id] = $size;
                    }
                }
            } else $this->error = isset($status[3]) ? $status[3] : 'Undefined error';
        } else {    
            fputs($this->_fs, "LIST $num\r\n");
            $status = $this->_get();
            if (isset($status[2]) && $status[2] == 'ok') {
                $t = explode(' ', $status[3]);
                $arr = $t[1];
            } else $this->error = isset($status[3]) ? $status[3] : 'Undefined error';
        }
        return $arr;
    }
    
    /**
    * Возвращает заголовки запрашиваемого сообщения.
    */
    function headers($id) {
        if (empty($this->_fs)) return FALSE;
        $this->error = '';
        
        $result = FALSE;
        fputs($this->_fs, 'TOP ' . $id . " 0\r\n");
        $status = $this->_get();
        if (isset($status[2]) && $status[2] == 'ok') {
            $result = $this->_get_other();
        } else $this->error = isset($status[3]) ? $status[3] : 'Undefined error';
        return $result;
    }
    
    /**
    * Возвращает заголовки запрашиваемого сообщения и несколько строк сообщения.
    */
    function part($id, $cnt = 2) {
        if (empty($this->_fs)) return FALSE;
        $this->error = '';
        
        $result = FALSE;
        fputs($this->_fs, "TOP $id $cnt\r\n");
        $status = $this->_get();
        if (isset($status[2]) && $status[2] == 'ok') {
            $result = array();
            $t      = $this->_get_other();
            if (strpos($t, "\r\n\r\n")) {
                list($result['headers'], $result['body']) = explode("\r\n\r\n", $t, 2);
            } else $result['headers'] = $t;
        } else $this->error = isset($status[3]) ? $status[3] : 'Undefined error';
        return $result;
    }
    
    /**
    * Возвращает запрашиваемое сообщение.
    */
    function retr($id) {
        if (empty($this->_fs)) return FALSE;
        $this->error = '';
        
        $result = FALSE;
        fputs($this->_fs, 'RETR ' . $id . "\r\n");
        $status = $this->_get();
        if (isset($status[2]) && $status[2] == 'ok') {
            $result = $this->_get_other();
        } else $this->error = isset($status[3]) ? $status[3] : 'Undefined error';
        return $result;
    }
    
    /**
    * Возвращает ответ сервера.
    *
    * если сервер возвращает несколько строк, соответствующая функция сама
    * должна вызвать _get_other() для получения оставшихся данных.
    * текущая фукнция предназначена для разбора первой строки ответа сервера.
    */
    private function _get() {
        $retr = fgets($this->_fs);
        preg_match('!^([\+-])([A-Za-z]+)(.*)!', $retr, $t);
        
        if (empty($t[2])) return FALSE;
        else {
            $t[3] = isset($t[3]) ? trim($t[3]) : '';
            $t[2] = strtolower($t[2]);
            return $t;
        }
    }
    
    /**
    * Возвращает ответ сервера.
    *
    * Признаком окончания передачи является символ точка.
    */
    private function _get_other() {
        $s = '';
        while ($t = fgets($this->_fs)) {
            if ($t === ".\r\n") break;
            # если первый символ ".", то он передался удвоенным
            # это делается для того, чтобы отличить конец передаваемых данных,
            # признаком которых и является точка
            if ($t[0] === '.') $t = substr($t, 1);
            $s .= $t;
        }
        return $s;
    }
    
}
